// ejercicio de estimador SISO no Observable

clear
xdel(winsid())
clc

// datos del problema

A1=12;
A2=45;
A3=15;
R1=120;
R2=360;
R3=72;
R4=300;
Kq=0.01;
Ts=10;

// matrices del modelo continuo

A=[-(1/R2+1/R1)/A1,0,1/(A1*R2); 1/(A2*R1),-1/(A2*R3),0; 1/(R2*A3),0,-(1/R2+1/R4)/A3];
B=[Kq/A1;0;0];
C=eye(3,3);
D=[0;0;0];

// Modelo de estado
P1=syslin("c",A,B,C,D)
// Matriz de observabilidad
V1=obsv_mat(A,C(3,:))
// Rango de la matriz V1 y Determinante
rank(V1)
det(V1)
// SIMULACIÓN
tiempo=[0:Ts:100000]; // Vector de Tiempos
u=ones(1,length(tiempo));
Y=csim(u,tiempo,P1); // Simulación
// GRÁFICO VARIABLES DE ESTADO A LAZO ABIERTO 
subplot(3,1,1)
plot2d(tiempo,Y(1,:),color("red"));
xgrid(color("gray"))
title('Respuesta de las variables de estado')
ylabel("Tanque nº 1");
subplot(3,1,2)
plot2d(tiempo,Y(2,:),color("blue"));
xgrid(color("gray"))
ylabel("Tanque nº 2");
subplot(3,1,3)
plot2d(tiempo,Y(3,:),color("scilabgreen4"));
xgrid(color("gray"))
ylabel("Tanque nº 3");


// Discretización del modelo continuo
clc;
xdel(winsid())
P2=dscr(P1,Ts)
Ad=P2.a;
Bd=P2.b;
Cd=P2.c(3,:);
Dd=P2.d(3,:);
V2=obsv_mat(Ad,Cd)
// Rango de la matriz V1 y Determinante
rank(V2)
det(V2)
// Función de Transferencia del modelo discreto
G2=ss2tf(P2)
zpk(G2)
/* Matriz de transformación para separar las variables
 observables y no observables.
 T=[C;CA;q1] */
clc;
q1=[0 1 0];
T=[V2(1,:); V2(2,:); q1]
P3=syslin("d",Ad,Bd,Cd,Dd);
// Modelo Transformado
P4=ss2ss(P3,T^-1)
autov=spec(Ad)

// Sub-sistema Observable
clc;
P5=syslin("d",P4.a(1:2,1:2),P4.b(1:2,1),P4.c(1,1:2),P4.d(1,1))
 Matriz de observabilidad y rango
V5=obsv_mat(P5)
rank(V5)

//  Elección de los autovalores
// constante de tiempo  500 seg

// AUTOVALORES.
CteT=50;
Psi= 1;
Wn = 1/(Psi*CteT);
p1 = -Psi*Wn+%i*Wn*sqrt(1-Psi^2)
p2 = -Psi*Wn-%i*Wn*sqrt(1-Psi^2)
// Autovalores discretos
polos=[exp(p1*Ts) exp(p2*Ts)]
// Vector del estimador observable
clc;
Ho=(ppol(P5.a',P5.c',polos))'
// Vector del estimador del modelo separado
Ht=[Ho; 0]
// Vector del estimador del modelo original
H=T^-1*Ht

Ae=Ad-H*Cd;
Be=[Bd,H];
Ce=eye(3,3);
De=zeros(3,2);
Xe0=[-0.5;-0.5;-0.5];

// RESPUESTA DEL ESTIMADOR 

At=[Ad,zeros(3,3);H*Cd,Ae]
Bt=[Bd;Bd]
Ct=eye(6,6)
Dt=zeros(6,1)
Xt=[Xe0;zeros(3,1)]
//Xt=zeros(6,1)
Pt=syslin("d",At,Bt,Ct,Dt,Xt)

// SIMULACIÓN
T = 0:Ts:20000;
u=ones(1,length(T));
Y=dsimul(Pt,u);
// GRÁFICO DEL ERROR
xdel(winsid());
subplot(3,1,1)
plot2d(T,Y(1,:),color("red"));
plot2d(T,Y(4,:),color("blue"));
xgrid(color("gray"))
title('Respuesta de las variables de estado')
ylabel("Altura 1");
subplot(3,1,2)
plot2d(T,Y(2,:),color("red"));
plot2d(T,Y(5,:),color("blue"));
xgrid(color("gray"))
ylabel("Altura 2");
subplot(3,1,3)
plot2d(T,Y(3,:),color("red"));
plot2d(T,Y(6,:),color("blue"))
xgrid(color("gray"))
ylabel("Altura 3");

